



IF NOT EXIST %LIBRARY_LIB%\amd.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\btf.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\camd.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\ccolamd.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\cholmod.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\colamd.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\cxsparse.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\klu.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\ldl.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\spqr.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
IF NOT EXIST %LIBRARY_LIB%\umfpack.lib exit 1
IF %ERRORLEVEL% NEQ 0 exit /B 1
exit /B 0
